#
# spec file for package kured
#
# Copyright (c) 2020 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#
# nodebuginfo


# Remove stripping of Go binaries.
%define __arch_install_post export NO_BRP_STRIP_DEBUG=true
# Project name when using go tooling.
%define project github.com/weaveworks/kured
# Project upstream commit.
%define commit f2a0f8e
Name:           kured
Version:        1.4.0
Release:        bp153.1.17
Summary:        Kubernetes daemonset to perform safe automatic node reboots
License:        Apache-2.0
Group:          System/Management
URL:            https://github.com/weaveworks/kured
Source:         %{name}-%{version}.tar.gz
Source1:        vendor.tar.xz
Patch:          systemctl-path.patch
Patch1:         kured-imagePullPolicy.patch
BuildRequires:  fdupes
BuildRequires:  go-go-md2man
BuildRequires:  golang(API) = 1.13
ExcludeArch:    s390

%description
Kured (KUbernetes REboot Daemon) is a Kubernetes daemonset that
performs safe automatic node reboots when the need to do so is
indicated by the package management system of the underlying OS.

- Watches for the presence of a reboot sentinel e.g. %{_localstatedir}/run/reboot-required

- Utilises a lock in the API server to ensure only one node reboots at a time

- Optionally defers reboots in the presence of active Prometheus alerts

- Cordons & drains worker nodes before reboot, uncordoning them after

%package k8s-yaml
Summary:        Kubernetes yaml file to run kured container
Group:          System/Management
BuildArch:      noarch

%description k8s-yaml
This package contains the yaml file requried to download and run the
kured container in a kubernetes cluster.

%prep
%setup -qa1
%patch -p1
%patch1 -p1

%build
# Build the binary.
export VERSION=%{version}
export COMMIT=%{commit}
go build \
   -mod vendor -buildmode=pie \
   -ldflags "-s -w -X main.gitCommit=$COMMIT -X main.version=$VERSION" \
   -o %{name} cmd/kured/*go

%install
# Install the binary.
install -D -m 0755 %{name} "%{buildroot}/%{_bindir}/%{name}"

# Build the man page from markdown documentation.
go-md2man -in README.md -out %{name}.1

# Install the man page.
install -D -m 0644 %{name}.1 "%{buildroot}/%{_mandir}/man1/%{name}.1"
rm %{name}.1

# Install provided yaml file to download and run the kured container
mkdir -p %{buildroot}%{_datadir}/k8s-yaml/kured
cat kured-rbac.yaml kured-ds.yaml > %{buildroot}%{_datadir}/k8s-yaml/kured/kured.yaml
chmod 644  %{buildroot}%{_datadir}/k8s-yaml/kured/kured.yaml
sed -i -e 's|image: .*|image: registry.opensuse.org/kubic/kured:%{version}|g' %{buildroot}%{_datadir}/k8s-yaml/kured/kured.yaml

%fdupes %{buildroot}

%files
%doc README.md
%license LICENSE
%{_bindir}/%{name}
%{_mandir}/man1/kured.1%{?ext_man}

%files k8s-yaml
%dir %{_datarootdir}/k8s-yaml
%dir %{_datarootdir}/k8s-yaml/kured
%{_datarootdir}/k8s-yaml/kured/kured.yaml

%changelog
* Mon May 18 2020 Thorsten Kukuk <kukuk@suse.com>
- kured-imagePullPolicy.patch: always update the image
* Sun May 17 2020 Thorsten Kukuk <kukuk@suse.com>
- systemctl-path.patch: last systemd update removed symlinks
  from /bin ...
* Mon May 11 2020 Thorsten Kukuk <kukuk@suse.com>
- Update to version 1.4.0
  - Updated kubectl, client-go, etc to k8s 1.17 (#127, #135)
  - Update to go 1.13 (#130)
  - print node id when commanding reboot (#134)
* Wed Apr 22 2020 Dominique Leuenberger <dimstar@opensuse.org>
- Fix build-dependency: we require golang(API) 1.12, not the exact
  go package version 1.12.
* Mon Mar  2 2020 Thorsten Kukuk <kukuk@suse.com>
- Update to version 1.3.0
  - Update k8s client tools to 1.15.x
  - Ad Slack channel name configuration
  - Add reboot window
- Obsoletes k8s-1.14.diff
- Remove kured-telemetrics.patch, chances that upstream accepts
  any third party code are nearly zero.
- Update vendor.tar.xz
* Mon Jun 24 2019 kukuk@suse.de
- k8s-1.14.diff: kubernetes 1.14.1 support from git
* Wed Jun  5 2019 kukuk@suse.de
- Fix path to image in manifest
* Wed May 22 2019 kukuk@suse.de
- Update to version 1.2.0
  - support newer kubernetes versions
- Adjust kured-telemetrics.patch
- Update vendor.tar.gz with recent versions
* Sat Apr  6 2019 kukuk@suse.de
- Enable building on s390x
* Thu Mar 28 2019 Jan Engelhardt <jengelh@inai.de>
- Combine %%setup calls.
* Thu Mar 28 2019 kukuk@suse.de
- kured-telemetrics.patch: add hooks for telemetrics data
- Renamed kured-yaml to kured-k8s-yaml to follow new policy
* Thu Feb 28 2019 kukuk@suse.de
- Change path in yaml file to point to official container image
* Fri Jan 18 2019 kukuk@suse.de
- Create a correct yaml file to download and run the kured container
  image in a kubernetes cluster
- Create new subpackage containing only the yaml file, so that
  people using the container don't need to install the not needed
  full package.
* Thu Nov 15 2018 Jeff Kowalczyk <jkowalczyk@suse.com>
- Update to kured 1.1.0
- Upstream bumped dependency on go1.10 via dependency k8s.io/client-go 0.9.0
  https://github.com/kubernetes/client-go
- Provide dependencies in separate vendor.tar.gz
- Improvements
  * RBAC support
  * Use the systemctl in the host mount namespace to effect reboots, reducing
    image size and eliminating the potential for incompatibility
  * Notify Slack on drain in addition to reboot
  * Pass through log output from invoked kubectl commands
  * Tolerate NoSchedule taint on node-role.kubernetes.io/master
  * Fixed reversal of daemonset name/namespace arguments and comments in the
    manifest
- Kubernetes Version Compatibility
  * The daemon image contains a 1.12.x k8s.io/client-go and kubectl binary for
    the purposes of maintaining the lock and draining worker nodes. Kubernetes
    aims to provide forwards & backwards compatibility of one minor version
    between client and server, so this should work on 1.11.x and 1.13.x.
  * Tested in minikube on 1.11.4, 1.12.1 & 1.13.0-alpha.2
  * Tested in production on 1.11.2 & 1.12.2
* Thu Sep 13 2018 jkowalczyk@suse.com
- Remove hardcoded GOARCH=amd64 and GOOS=linux
- Revise go build arg -ldflags and add -buildmode=pie
- Together these fix rpmlint warnings:
  * position-independent-executable-suggested
  * statically-linked-binary
- Upstream kured project code imports package as 'context'. Bump BuildRequires
  to go1.7 wherein import path for package context graduates from
  'golang.org/x/net/context' to the standard library as 'context'.
  https://golang.org/doc/go1.7#context
- Bump release number
* Wed Sep 12 2018 jkowalczyk@suse.com
- Initial packaging of upstream master branch @ 5731b98 (tagged 1.0.0 + 24)
- Include 24 commits since release 1.0.0 updating kubernetes version support
- Dependency sources vendored via dep ensure per upstream build instructions
- Man page converted from README.md, some HTML formatting artifacts present
- rpmlint warning: position-independent-executable-suggested
  * go1.11 currently in review status supports option -buildmode=pie
- rpmlint warning: statically-linked-binary
  * Go binaries are generally statically linked
